import base64
from Crypto.Cipher import AES

'''
采用AES对称加密算法
'''


class Encryption(object):

    sk = 'sdf8g4edf809p2fssdai'

    @classmethod
    def __add_to_16(cls, value):
        while len(value) % 16 != 0:
            value += '\0'
        return str.encode(value)

    @classmethod
    def encrypt_key(cls, key):
        aes = AES.new(cls.__add_to_16(cls.sk), AES.MODE_ECB)
        encrypt_aes = aes.encrypt(cls.__add_to_16(key))
        encrypted_text = str(base64.encodebytes(encrypt_aes), encoding='utf-8')  # 执行加密并转码返回bytes
        if encrypted_text.endswith('\n'):
            encrypted_text = encrypted_text.split('\n')[0]
        return encrypted_text

    @classmethod
    def decrypt_key(cls, key):
        aes = AES.new(cls.__add_to_16(cls.sk), AES.MODE_ECB)
        base64_decrypted = base64.decodebytes(key.encode(encoding='utf-8'))
        decrypted_text = str(aes.decrypt(base64_decrypted), encoding='utf-8').replace('\0', '')
        return decrypted_text

    @classmethod
    def encryt_dict(cls, config_map, ignore=None):
        # ignore 为忽略的字段，默认值是none，值为list，如['a', 'b']
        _res = {}
        for _k, _v in config_map.items():
            if ignore and _k in ignore:
                _res[_k] = _v
                continue
            _res[_k] = cls.encrypt_key(_v)
        return _res

    @classmethod
    def decrypt_dict(cls, config_map, ignore=None):
        # ignore 为忽略的字段，默认值是none，值为list，如['a', 'b']
        _res = {}
        for _k, _v in config_map.items():
            if ignore and _k in ignore:
                _res[_k] = _v
                continue
            try:
                _res[_k] = cls.decrypt_key(_v)
            except:
                _res[_k] = _v
        return _res


import multiprocessing
import time
from sense_core import RabbitmqFactory


class ThreadAutoFactory(object):
    def __init__(self, queue, label, threshold=100, work=3, max_work=10, monitor=60):
        """
        :param queue:      消费队列
        :param label:      消费的mq
        :param auto_large: 伸缩最大阈值，当msg_num > work * auto_large 则work数加1
        :param auto_small: 伸缩最小阈值，当msg_num < work * auto_small 则work数减1
        :param work:       初始work数，也即最小work数
        :param max_work:   最大work数
        :param monitor:    监控间隔时间，每多少时间进行一次work和msg检查，进行auto scale
        """
        self.queue = queue
        self.threshold = threshold
        self.work = work
        self.max_work = max_work
        self.min_work = work
        self.mq = RabbitmqFactory(label)
        self.thread_list = []
        self.monitor = monitor
        self.msg_num = self.get_mq_queue_num()

    def get_mq_queue_num(self):
        _code, queue_details = self.mq.list_queues(dict_type=True)
        return queue_details.get(self.queue, 0)

    def auto_scale_num(self):
        # 获取队列长度，并进行work数变更
        self.msg_num = self.get_mq_queue_num()
        if self.msg_num > self.threshold * len(self.thread_list):
            self.work += 1
            if self.work > self.max_work:
                self.work = self.max_work
        else:
            self.work -= 1
            if self.work < self.min_work:
                self.work = self.min_work
        return self.work

    def execute(self, function, params=()):
        while True:
            # 检测线程存活情况,将死掉的进程去除
            for _thread in self.thread_list:
                if not _thread.is_alive():
                    self.thread_list.remove(_thread)
            # work 扩张
            if self.work > len(self.thread_list):
                _append = self.work - len(self.thread_list)
                _new_add_thread = []
                for num in range(0, _append):
                    _new_add_thread.append(multiprocessing.Process(target=function, args=params))
                for _thread in _new_add_thread:
                    _thread.start()
                self.thread_list.extend(_new_add_thread)
            time.sleep(self.monitor)
            self.work = self.auto_scale_num()
            # work 缩减
            if self.work < len(self.thread_list):
                _del_num = len(self.thread_list) - self.work
                for _thread in self.thread_list[:_del_num]:
                    _thread.terminate()
                    _thread.join()
                    self.thread_list.remove(_thread)


import os
from setuptools import setup

requirements = [
    "pymongo>=3.7",
    "python-decouple>=3.1",
    "redis>2.10",
    "pycrypto==2.6.1",
    "elasticsearch-dsl==6.3.1",
    "pika==0.12",
    "shortuuid",
    "requests",
    "threadpool>=1.3.2",
    "python-dateutil>=2.7",
    "sqlalchemy>=1.2"
]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='sense-core',
    version='0.2.62',
    packages=[
            "sense_core",
    ],
    license='BSD License',  # example license
    description='sense core',
    install_requires=requirements,
    long_description_content_type="text/markdown",
    url='',
    author='kafka0102',
    author_email='yujianjia@sensedeal.ai',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)